from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from sqlalchemy import *
from sqlalchemy.orm import create_session, sessionmaker, relationship, backref
from sqlalchemy.ext.declarative import declarative_base
from geoalchemy import postgis, GeometryColumn, Point, MultiPolygon
import simplejson as json

engine = create_engine('postgresql://abs:abs@localhost/abs', echo=False)
Session = sessionmaker(bind=engine)
session = Session()

metadata = MetaData(bind=engine)
Base = declarative_base(metadata=metadata)


class medicare(Base):
    __tablename__ = 'medicare'
    __table_args__ = {'autoload': True, 'schema': 'govhack_2013'}
    geom = GeometryColumn(Point())


class lga(Base):
    __tablename__ = 'lga'
    __table_args__ = {'autoload': True, 'schema': 'asgs_2011'}
    geom = GeometryColumn(MultiPolygon(1,srid=4326))


class rent(Base):
    __tablename__ = 'rent'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    lga = Column(Integer, ForeignKey('asgs_2011.lga.code'))
    lga_code = Column(Integer)
    rank= Column(Integer)
    house_2b = Column(Integer)
    house_3b = Column(Integer)
    flat_1b = Column(Integer)
    flat_2b = Column(Integer)
    avg_all = Column(Integer)
    lga_obj = relationship("lga", backref="rent")


class tz_nsw_2011(Base):
    __tablename__ = 'tz_nsw_2011'
    __table_args__ = {'autoload': True, 'schema': 'govhack_2013'}
    geom = GeometryColumn(MultiPolygon(1,srid=4326))


class universities(Base):
    __tablename__ = 'universities'
    __table_args__ = {'autoload': True, 'schema': 'govhack_2013'}
    geom = GeometryColumn(MultiPolygon(1,srid=4326))


class uni_tz_commute(Base):
    __tablename__ = 'uni_tz_commute'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    tz = Column(Integer, ForeignKey('govhack_2013.tz_nsw_2011.gid'))
    uni = Column(Integer, ForeignKey('govhack_2013.universities.gid'))
    duration = Column(Integer)
    tz_obj = relationship("tz_nsw_2011", backref="get_uni_tz_commute_tz")
    uni_obj = relationship("universities", backref="get_uni_tz_commute_uni")

class score(Base):
    __tablename__ = 'score'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    tz = Column(Integer, ForeignKey('govhack_2013.tz_nsw_2011.gid'))
    uni = Column(Integer, ForeignKey('govhack_2013.universities.gid'))
    crime = Column(Integer)
    travel = Column(Integer)
    rent = Column(Integer)
    service = Column(Integer)
    total = Column(Integer)
    tz_obj = relationship("tz_nsw_2011", backref="score")
    uni_obj = relationship("universities", backref="score")


class services(Base):
    __tablename__ = 'services'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    tz = Column(Integer, ForeignKey('govhack_2013.tz_nsw_2011.gid'))
    supermarket = Column(Integer)
    health = Column(Integer)
    library = Column(Integer)
    tz_obj = relationship("tz_nsw_2011", backref="services")


def generate_geojson(request, transport):
    #request.GET.get('gid', '')

    # Get MB listing for Willoughby
    #mb_list = session.query(mb).filter(mb.lga == '18250')

    geom_list = []

    #q=session.query(mb, medicare).filter(mb.geom.within_distance(medicare.geom, .005))
    #results = session.query(score).filter(uni_tz_commute.uni == 2)\
    #    .filter(score.tz_obj.get_uni_tz_commute_tz.duration > 50).filter(score.tz_obj.get_uni_tz_commute_tz.duration <3600)
    results = session.query(score).filter(score.uni == 2)\
        .filter(score.travel != None)

    #for meshblock in mb_list:
    for result in results:
        #geom_list.append(session.scalar(postgis.pg_functions.geojson(result.tz_obj.geom)))
        #str_geom = '{"type":"MultiPolygon","coordinates":[[[[151.207320960025498,-33.865170382985617],[151.207203615483564,-33.865130700289555],[151.206888416264121,-33.86510898104212],[151.206571071805541,-33.865086504073126],[151.206457856273033,-33.86508731817576],[151.205620671502089,-33.865087059366012],[151.205515936367192,-33.864288931851696],[151.205810943836184,-33.864332314428374],[151.206003167755995,-33.864335940421697],[151.206484320163014,-33.864347040531378],[151.206574207889844,-33.86435871355458],[151.206995327768141,-33.864367760092186],[151.207011040400118,-33.864358399724061],[151.207412415917503,-33.864366780100056],[151.207398016535791,-33.864545878858287],[151.207383967727225,-33.864627870240504],[151.207368416074587,-33.864715930321125],[151.207320960025498,-33.865170382985617]]]]}'
        coord_json = session.scalar(postgis.pg_functions.geojson(result.tz_obj.geom))
        temp = json.loads(coord_json)
        temp['duration'] = result.travel
        temp['total'] = result.total
        temp['tz_name11'] = result.tz_obj.tz_name11
        temp['tz_code11'] = result.tz_obj.tz_code11
        geom_list.append(json.dumps(temp))

    str_geom = ', '.join(geom_list)
    return render_to_response('geojson.js', {"data": str_geom},
                              mimetype="application/json")

def get_tz_stats(request, tz):
    tz_obj = session.query(tz_nsw_2011).filter(tz_nsw_2011.tz_code11 == tz).first()
    services_obj = session.query(services).filter(services.tz == tz).first()
    rent_obj = session.query(rent).filter(rent.lga == tz_obj.lga).first()
    tz_str = "<li>Nearby Healthcare: %s</li><li>Nearby Libraries: %s</li><li>Nearby Supermarkets: %s</li>" % (services_obj.health, services_obj.library, services_obj.supermarket)
    rent_str = "<li>1 Bedroom Flat: $%s</li><li>2 Bedroom Flat: $%s</li><li>2 Bedroom House: $%s</li><li>3 Bedroom House: $%s</li>" % (rent_obj.flat_1b, rent_obj.flat_2b, rent_obj.house_2b, rent_obj.house_3b)
    return HttpResponse(tz_str + rent_str) #content_type="application/json"

def get_tz_rent(request, tz):
    tz_obj = session.query(tz_nsw_2011).filter(tz_nsw_2011.tz_code11 == tz).first()
    rent_obj = session.query(rent).filter(rent.lga == tz_obj.lga).first()
    return HttpResponse(tz_str) #content_type="application/json"

def livewhere(request):
    gid = 2
    uni = session.query(universities).filter(universities.gid == gid).first()
    data = {
        "university": uni
    }
    return render_to_response('map.html', data, context_instance=RequestContext(request))


def safe(request):
    return render_to_response('safe.html')

def friends(request):
    return render_to_response('meetfriends.html')

def about(request):
    return render_to_response('about.html')

def essential(request):
    return render_to_response('essential.html')

def index(request):
    return render_to_response('index.html')