__author__ = 'kelvinn'
from sqlalchemy import *
from sqlalchemy.orm import create_session, sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from geoalchemy import postgis
from sqlalchemy.orm import relationship, backref
from geoalchemy import *
import simplejson as json
import urllib2
import os
import csv
import itertools

#Create and engine and get the metadata
#Base = declarative_base()

engine = create_engine('postgresql://abs:abs@localhost/abs', echo=False) #change here if want to view echo
Session = sessionmaker(bind=engine)
session = Session()

metadata = MetaData(bind=engine)
Base = declarative_base(metadata=metadata)


class sa2(Base):
    __tablename__ = 'sa2'
    __table_args__ = {'autoload': True, 'schema': 'asgs_2011'}
    geom = GeometryColumn(MultiPolygon(1,srid=4326))


class tz_nsw_2011(Base):
    __tablename__ = 'tz_nsw_2011'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    sa2 = Column(Integer, ForeignKey('asgs_2011.sa2.code'))
    lga = Column(Integer, ForeignKey('asgs_2011.lga.code'))
    sa2_main11 = Column(Integer)
    tz_code11 = Column(Integer)
    tz_name11 = Column(String(100))
    sa2_obj = relationship("sa2", backref="tz")
    lga_obj = relationship("lga", backref="tz")
    geom = GeometryColumn(MultiPolygon(1,srid=4326))


class universities(Base):
    __tablename__ = 'universities'
    __table_args__ = {'autoload': True, 'schema': 'govhack_2013'}
    geom = GeometryColumn(MultiPolygon(1,srid=4326))


class libraries(Base):
    __tablename__ = 'libraries'
    __table_args__ = {'autoload': True, 'schema': 'govhack_2013'}
    geom = GeometryColumn(MultiPolygon(1,srid=4326))


class lga(Base):
    __tablename__ = 'lga'
    __table_args__ = {'autoload': True, 'schema': 'asgs_2011'}
    geom = GeometryColumn(MultiPolygon(1,srid=4326))


class crime(Base):
    __tablename__ = 'crime'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    lga = Column(Integer, ForeignKey('asgs_2011.lga.code'))
    rank_2011 = Column(Integer)
    lga_obj = relationship("lga", backref="crime")


class planet_osm_point(Base):
    __tablename__ = 'planet_osm_point'
    __table_args__ = {'autoload': True, 'schema': 'public'}
    way = GeometryColumn(Point(1,srid=4326))

class rent(Base):
    __tablename__ = 'rent'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    lga = Column(Integer, ForeignKey('asgs_2011.lga.code'))
    lga_code = Column(Integer)
    rank= Column(Integer)
    lga_obj = relationship("lga", backref="rent")

class uni_tz_commute(Base):
    __tablename__ = 'uni_tz_commute'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    tz = Column(Integer, ForeignKey('govhack_2013.tz_nsw_2011.gid'))
    uni = Column(Integer, ForeignKey('govhack_2013.universities.gid'))
    duration = Column(Integer)
    tz_obj = relationship("tz_nsw_2011", backref="uni_tz_commute")
    uni_obj = relationship("universities", backref="uni_tz_commute")

class score(Base):
    __tablename__ = 'score'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    tz = Column(Integer, ForeignKey('govhack_2013.tz_nsw_2011.gid'))
    uni = Column(Integer, ForeignKey('govhack_2013.universities.gid'))
    crime = Column(Integer)
    travel = Column(Integer)
    rent = Column(Integer)
    service = Column(Integer)
    total = Column(Integer)
    tz_obj = relationship("tz_nsw_2011", backref="score")
    uni_obj = relationship("universities", backref="score")

class services(Base):
    __tablename__ = 'services'
    __table_args__ = {'schema': 'govhack_2013'}
    gid = Column(Integer, primary_key=True)
    tz = Column(Integer, ForeignKey('govhack_2013.tz_nsw_2011.gid'))
    supermarket = Column(Integer)
    health = Column(Integer)
    library = Column(Integer)
    tz_obj = relationship("tz_nsw_2011", backref="services")

def get_commute_score(duration):
    if 0<= duration <=700:
        return 100
    elif 701<= duration <=1400:
        return 75
    elif 1401<= duration <=2100:
        return 50
    elif 2101<= duration <=2800:
        return 25
    else:
        return None





#one_obj = session.query(uni_tz_commute).filter(uni_tz_commute.uni == "1")\
#    .filter(uni_tz_commute.duration > 50).filter(uni_tz_commute.duration <3600).first()



# Build the score table template. Only run once!
"""
uni_list = session.query(universities)
tz_list = session.query(tz_nsw_2011)
for uni_obj in uni_list:
    for tz_obj in tz_list:
        c = score()
        c.tz = tz_obj.gid
        c.uni = uni_obj.gid
        session.add(c)
    session.commit()
"""

"""
uni_list = session.query(universities)
for uni_obj in uni_list:
    travel_result_list = session.query(uni_tz_commute).filter(uni_tz_commute.uni == uni_obj.gid)\
        .filter(uni_tz_commute.duration > 1).filter(uni_tz_commute.duration <3600)
    for tz_commute_obj in travel_result_list:
        try:
            tz_commute_obj.tz_obj.score[0].travel = get_commute_score(tz_commute_obj.duration)
            session.add(tz_commute_obj)
        except Exception,e: print str(e)
    session.commit()
"""



# Compute the crime score for any given TZ
"""
tz_result_list = session.query(tz_nsw_2011)
for tz_one in tz_result_list:
    try:
        rank = int((float(tz_one.lga_obj.crime[0].rank_2011)/153.0)*100)
        for i in tz_one.score:
            i.crime = rank
            session.add(i)
    except Exception,e: print str(e)
session.commit()
"""

"""
tz_result_list = session.query(tz_nsw_2011)
for tz_one in tz_result_list:
    try:
        rank = int((float(tz_one.lga_obj.rent[0].rank)/51.0)*100)
        for i in tz_one.score:
            i.rent = rank
            session.add(i)
    except Exception,e: print str(e)
session.commit()
"""
"""
tz_result_list = session.query(tz_nsw_2011)
for tz_obj in tz_result_list:
    c = services()
    c.tz = tz_obj.gid
    session.add(c)
session.commit()
"""

"""
shopping_list = session.query(tz_nsw_2011, planet_osm_point).filter(tz_nsw_2011.geom.within_distance(planet_osm_point.way, .002)).filter(planet_osm_point.shop is not None)
for item in shopping_list:
    service_obj = session.query(services).filter(services.tz == item.tz_nsw_2011.gid).first()
    service_obj.supermarket = 1
    session.add(service_obj)
session.commit()
"""

"""
health_list = session.query(tz_nsw_2011, planet_osm_point).filter(tz_nsw_2011.geom.within_distance(planet_osm_point.way, .005))\
.filter(or_(planet_osm_point.amenity == "hospital", planet_osm_point.amenity == "doctors", planet_osm_point.amenity == "clinic"))
for item in health_list:
    service_obj = session.query(services).filter(services.tz == item.tz_nsw_2011.gid).first()
    service_obj.health = 1
    session.add(service_obj)
session.commit()
"""

"""
library_list = session.query(tz_nsw_2011, libraries).filter(tz_nsw_2011.geom.within_distance(libraries.geom, .003))
for item in library_list:
    service_obj = session.query(services).filter(services.tz == item.tz_nsw_2011.gid).first()
    service_obj.library = 1
    session.add(service_obj)
session.commit()
"""
"""
service_list = session.query(services)
for service_one in service_list:
    try:
        rank = [service_one.health, service_one.library, service_one.supermarket].count(1) # hack to not throw err
        for i in service_one.tz_obj.score:
            if rank == 3:
                i.service = 100
            elif rank == 2:
                i.service = 66
            elif rank == 1:
                i.service = 33
            else:
                i.service = 0
            session.add(i)
    except Exception,e: print str(e)
session.commit()
"""
"""
score_list = session.query(score)
for score_one in score_list:
    try:
        total = 0
        i = 0
        to_count = []
        if score_one.rent is not None:
            to_count.append(score_one.rent)
        if score_one.travel is not None:
            to_count.append(score_one.travel)
        if score_one.crime is not None:
            to_count.append(score_one.crime)
        if score_one.service is not None:
            to_count.append(score_one.service)
        score_one.total = sum(to_count)/len(to_count)
        session.add(score_one)
    except Exception,e: print str(e)
session.commit()
"""

score_list = session.query(score)
for score_one in score_list:
    uni_tz_commute_obj = session.query(uni_tz_commute).filter(uni_tz_commute.uni == score_one.uni).filter(uni_tz_commute.tz == score_one.tz).first()
    if uni_tz_commute_obj:
        score_one.travel = get_commute_score(uni_tz_commute_obj.duration)
        session.add(score_one)
session.commit()

#travel_result_list = session.query(uni_tz_commute, crime).filter(uni_tz_commute.uni == "1")\
#    .filter(uni_tz_commute.duration > 50).filter(uni_tz_commute.duration <3600)

